@@ -240,7 +240,7 @@ class cdrecord:
 
     def __cdrecord_read_version(self):
         """ Reads the version string by executing "cdrecord -version". """
-        output = cmdoutput("%s -version 2>&1" % self.__cdrecord_command, strip = TRUE)
+        output = cmdoutput("%s -version" % self.__cdrecord_command, strip = TRUE)
         versionLine = output[0]
         splitted = split(versionLine, " ")
         version = split(splitted[1], ".")
@@ -364,13 +364,17 @@ class readcd:
 
 class cdda2wav:
 
-    def __init__(self, bus, target, lun, loglevel = LOGLEVEL_NORMAL):
+    def __init__(self, bus, target, lun, loglevel = LOGLEVEL_NORMAL, use_atapi = USE_ATAPI):
         self.__cdda2wav_logger = Logger().get_instance(self)
         self.__cdda2wav_logger.set_loglevel(loglevel)
         self.__cdda2wav_command = which("cdda2wav")
-        self.__cdda2wav_bus = int(bus)
-        self.__cdda2wav_target = int(target)
-        self.__cdda2wav_lun = int(lun)
+        if (type(bus) == StringType): bus = atoi(bus)
+        if (type(target) == StringType): target = atoi(target)
+        if (type(lun) == StringType): lun = atoi(lun)
+        self.__cdda2wav_device_string = "dev="
+        if (use_atapi == TRUE):
+            self.__cdda2wav_device_string = "%sATAPI:" % self.__cdda2wav_device_string
+        self.__cdda2wav_device_string = "%s%d,%d,%d" % (self.__cdda2wav_device_string, bus, target, lun)
         self.__cdda2wav_read_version()
 
     def __cdda2wav_read_version(self):
@@ -390,12 +394,12 @@ class cdda2wav:
         return "%d.%d" % (self.__cdda2wav_version_major, self.__cdda2wav_version_minor)
 
     def get_track_info(self):
-        command = "%s -D %d,%d,%d -N -J -g -Q" % (self.__cdda2wav_command, self.__cdda2wav_bus, self.__cdda2wav_target, self.__cdda2wav_lun)
+        command = "%s -N -J -g -Q" % (self.__cdda2wav_command)
         if (self.__cdda2wav_version_major < 2):
             command = "%s -v 2" % command
         else:
             command = "%s -v toc" % command
-        command = "%s 2>&1" % command
+        command = "%s %s 2>&1" % (command, self.__cdda2wav_device_string)
         self.__cdda2wav_logger.debug("Executing %s" % command)
         output = cmdoutput(command)
         tracklengths = {}
